# Auto-generated stub file for IDE autocomplete support.
# Do not edit manually - regenerate with: python scripts/generate_stubs.py

from dataclasses import dataclass
from typing import Any, ClassVar, List, Optional, TypeVar, Generic
from typing_extensions import Self
from uuid import UUID
import weakref

P = TypeVar('P')

from abc import ABC, abstractmethod
from pathlib import Path

class Parser(ABC):
    def parse_inputs(self, sources: Iterable[ParserInput], relative_to: Optional[Path], ctx: ExecutionContext) -> Iterable[SourceFile]: ...
    def accept(self, path: Path) -> bool: ...
    def source_path_from_source_text(self, prefix: Path, source_code: str) -> Path: ...
    def parse(self, source_files: Iterable[Path], relative_to: Optional[Path], ctx: ExecutionContext) -> Iterable[SourceFile]: ...
    def parse_strings(self) -> Iterable[SourceFile]: ...
    def accept_input(self, parser_input: ParserInput) -> bool: ...
    def accepted_inputs(self, inputs: Iterable[ParserInput]) -> Iterable[ParserInput]: ...
    def get_charset(self, ctx: ExecutionContext) -> str: ...

class ParserBuilder(ABC):
    @property
    def source_file_type(self) -> type: ...
    @property
    def dsl_name(self) -> Optional[str]: ...
    def build(self) -> Parser: ...

@dataclass(frozen=True)
class ParserInput:
    _path: Path
    _file_attributes: Optional[FileAttributes]
    _synthetic: bool
    _source: Callable[[], IO[Any]]

    def replace(self, **kwargs: Any) -> Self: ...

    @property
    def path(self) -> Path: ...
    @property
    def file_attributes(self) -> Optional[FileAttributes]: ...
    @property
    def synthetic(self) -> bool: ...
    @property
    def source(self) -> Callable[[], IO[Any]]: ...

@dataclass(frozen=True)
class ParseError(SourceFile):
    _id: UUID
    _markers: Markers
    _source_path: Path
    _file_attributes: Optional[FileAttributes]
    _charset_name: Optional[str]
    _charset_bom_marked: bool
    _checksum: Optional[Checksum]
    _text: str
    _erroneous: Optional[SourceFile]

    def replace(self, **kwargs: Any) -> Self: ...

    @classmethod
    def build(cls, parser: 'Parser', input: ParserInput, relative_to: Optional[Path], ctx: ExecutionContext, exception: Exception, erroneous: Optional[SourceFile]) -> 'ParseError': ...

    @property
    def id(self) -> UUID: ...
    @property
    def markers(self) -> Markers: ...
    @property
    def source_path(self) -> Path: ...
    @property
    def file_attributes(self) -> Optional[FileAttributes]: ...
    @property
    def charset_name(self) -> Optional[str]: ...
    @property
    def charset_bom_marked(self) -> bool: ...
    @property
    def checksum(self) -> Optional[Checksum]: ...
    @property
    def text(self) -> str: ...
    @property
    def erroneous(self) -> Optional[SourceFile]: ...

    def with_id(self, id: UUID) -> 'ParseError': ...
    def with_markers(self, markers: Markers) -> 'ParseError': ...
    def with_source_path(self, source_path: Path) -> 'ParseError': ...
    def with_file_attributes(self, file_attributes: Optional[FileAttributes]) -> 'ParseError': ...
    def with_charset_name(self, charset_name: Optional[str]) -> 'ParseError': ...
    def with_charset_bom_marked(self, charset_bom_marked: bool) -> 'ParseError': ...
    def with_checksum(self, checksum: Optional[Checksum]) -> 'ParseError': ...
    def with_text(self, text: str) -> 'ParseError': ...
    def with_erroneous(self, erroneous: Optional[SourceFile]) -> 'ParseError': ...
    def printer(self, cursor: Cursor) -> TreeVisitor[Tree, PrintOutputCapture[P]]: ...
    def is_acceptable(self, v: TreeVisitor[Any, P], p: P) -> bool: ...
    def accept(self, v: TreeVisitor[Any, P], p: P) -> Optional[Any]: ...



# Copyright 2025 the original author or authors.
#
# Licensed under the Moderne Source Available License (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://docs.moderne.io/licensing/moderne-source-available-license
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Precondition helpers that delegate to Java search recipes via RPC.

These functions provide preconditions for Python recipes that need to check
if source files match certain criteria before applying transformations.
When RPC is available, they delegate to the Java implementations for efficiency.

Example:
    from rewrite.python.preconditions import uses_type, uses_method

    class MyRecipe(Recipe):
        def preconditions(self):
            return uses_type("datetime.datetime")
"""

from typing import Any, Optional, Union

from rewrite.rpc.java_recipe import prepare_java_recipe, PreparedJavaRecipe


def has_source_path(file_pattern: str) -> PreparedJavaRecipe:
    """
    Create a precondition that matches source files by path pattern.

    Delegates to org.openrewrite.FindSourceFiles.

    Args:
        file_pattern: Glob pattern to match file paths (e.g., "**/*.py")

    Returns:
        A prepared recipe that can be used as a precondition
    """
    return prepare_java_recipe("org.openrewrite.FindSourceFiles", {
        "filePattern": file_pattern
    })


def uses_method(method_pattern: str, match_overrides: bool = False) -> PreparedJavaRecipe:
    """
    Create a precondition that matches files using a specific method.

    Delegates to org.openrewrite.java.search.HasMethod.

    Args:
        method_pattern: Method pattern to search for (e.g., "datetime.datetime utcnow()")
        match_overrides: Whether to also match method overrides

    Returns:
        A prepared recipe that can be used as a precondition
    """
    return prepare_java_recipe("org.openrewrite.java.search.HasMethod", {
        "methodPattern": method_pattern,
        "matchOverrides": match_overrides
    })


def uses_type(fully_qualified_type: str, check_assignability: bool = False) -> PreparedJavaRecipe:
    """
    Create a precondition that matches files using a specific type.

    Delegates to org.openrewrite.java.search.HasType.

    Args:
        fully_qualified_type: Fully qualified type name (e.g., "datetime.datetime")
        check_assignability: Whether to check type assignability

    Returns:
        A prepared recipe that can be used as a precondition
    """
    return prepare_java_recipe("org.openrewrite.java.search.HasType", {
        "fullyQualifiedTypeName": fully_qualified_type,
        "checkAssignability": check_assignability
    })


def find_methods(method_pattern: str, match_overrides: bool = False) -> PreparedJavaRecipe:
    """
    Create a recipe that finds and marks methods matching a pattern.

    Delegates to org.openrewrite.java.search.FindMethods.

    Args:
        method_pattern: Method pattern to search for
        match_overrides: Whether to also match method overrides

    Returns:
        A prepared recipe that marks matching methods
    """
    return prepare_java_recipe("org.openrewrite.java.search.FindMethods", {
        "methodPattern": method_pattern,
        "matchOverrides": match_overrides
    })


def find_types(fully_qualified_type: str) -> PreparedJavaRecipe:
    """
    Create a recipe that finds and marks usages of a type.

    Delegates to org.openrewrite.java.search.FindTypes.

    Args:
        fully_qualified_type: Fully qualified type name to find

    Returns:
        A prepared recipe that marks matching type usages
    """
    return prepare_java_recipe("org.openrewrite.java.search.FindTypes", {
        "fullyQualifiedTypeName": fully_qualified_type
    })



# Copyright 2025 the original author or authors.
#
# Licensed under the Moderne Source Available License (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://docs.moderne.io/licensing/moderne-source-available-license
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
AddLiteralMethodArgument recipe for Python that delegates to Java's AddLiteralMethodArgument.

This recipe adds a literal argument to method invocations matching a pattern.
"""

from typing import Any, Optional

from rewrite import ExecutionContext, Recipe, TreeVisitor, option
from rewrite.rpc.java_recipe import prepare_java_recipe, JavaRecipeVisitor, PreparedJavaRecipe


class AddLiteralMethodArgument(Recipe):
    """
    Add a literal argument to method invocations matching a pattern.

    This recipe delegates to Java's org.openrewrite.java.AddLiteralMethodArgument recipe.

    Example:
        AddLiteralMethodArgument(
            method_pattern="datetime.datetime now()",
            argument_index=0,
            literal="datetime.UTC"
        )
    """

    method_pattern: str = option(
        display_name="Method pattern",
        description="A method pattern that matches method invocations to modify.",
        example="datetime.datetime now()"
    )

    argument_index: int = option(
        display_name="Argument index",
        description="The zero-based position where to insert the argument.",
        example="0"
    )

    literal: str = option(
        display_name="Literal",
        description="The literal value to add as an argument.",
        example="datetime.UTC"
    )

    def __init__(
        self,
        method_pattern: str,
        argument_index: int,
        literal: str
    ):
        self.method_pattern = method_pattern
        self.argument_index = argument_index
        self.literal = literal
        self._prepared_recipe: Optional[PreparedJavaRecipe] = None

    @property
    def name(self) -> str:
        return "org.openrewrite.python.AddLiteralMethodArgument"

    @property
    def display_name(self) -> str:
        return "Add literal method argument"

    @property
    def description(self) -> str:
        return "Add a literal argument to method invocations matching a pattern."

    def editor(self) -> TreeVisitor[Any, ExecutionContext]:
        if self._prepared_recipe is None:
            self._prepared_recipe = prepare_java_recipe(
                "org.openrewrite.java.AddLiteralMethodArgument",
                {
                    "methodPattern": self.method_pattern,
                    "argumentIndex": self.argument_index,
                    "literal": self.literal
                }
            )

        return JavaRecipeVisitor(self._prepared_recipe)



# Copyright 2026 the original author or authors.
# <p>
# Licensed under the Moderne Source Available License (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# <p>
# https://docs.moderne.io/licensing/moderne-source-available-license
# <p>
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Recipe to migrate deprecated datetime.utcnow() and datetime.utcfromtimestamp() calls."""

from typing import Any, List, Optional
from uuid import uuid4

from rewrite import ExecutionContext, Recipe, TreeVisitor
from rewrite.category import CategoryDescriptor
from rewrite.decorators import categorize
from rewrite.java import J
from rewrite.java.support_types import JContainer, JLeftPadded, JRightPadded, Space
from rewrite.java.tree import Empty, FieldAccess, Identifier, Import, MethodInvocation
from rewrite.markers import Markers
from rewrite.marketplace import Python
from rewrite.python.template import template
from rewrite.python.tree import CompilationUnit, MultiImport
from rewrite.python.visitor import PythonVisitor

_Migration = [*Python, CategoryDescriptor(display_name="Migration")]


def _get_import_style(cu: CompilationUnit) -> Optional[str]:
    """Determine how datetime is imported.

    Returns:
        "from" if `from datetime import datetime` is used,
        "direct" if `import datetime` is used,
        None if neither is found.
    """
    for stmt in cu.statements:
        if not isinstance(stmt, MultiImport):
            continue
        from_ = stmt.from_
        if from_ is not None:
            # from X import Y — check if X is "datetime"
            if isinstance(from_, Identifier) and from_.simple_name == "datetime":
                for imp in stmt.names:
                    if imp.qualid.name.simple_name == "datetime":
                        return "from"
            elif isinstance(from_, FieldAccess) and from_.name.simple_name == "datetime":
                for imp in stmt.names:
                    if imp.qualid.name.simple_name == "datetime":
                        return "from"
        else:
            # import X — check if any name is "datetime"
            for imp in stmt.names:
                if imp.qualid.name.simple_name == "datetime":
                    return "direct"
    return None


def _has_timezone_import(cu: CompilationUnit) -> bool:
    """Check if `timezone` is already imported from `datetime`."""
    for stmt in cu.statements:
        if not isinstance(stmt, MultiImport):
            continue
        from_ = stmt.from_
        if from_ is None:
            continue
        if isinstance(from_, Identifier) and from_.simple_name == "datetime":
            for imp in stmt.names:
                if imp.qualid.name.simple_name == "timezone":
                    return True
    return False


def _is_datetime_receiver(select: Optional[J], import_style: str) -> bool:
    """Check if the select expression matches the expected datetime receiver."""
    if select is None:
        return False
    if import_style == "from":
        return isinstance(select, Identifier) and select.simple_name == "datetime"
    elif import_style == "direct":
        if not isinstance(select, FieldAccess):
            return False
        return (
            isinstance(select.target, Identifier)
            and select.target.simple_name == "datetime"
            and select.name.simple_name == "datetime"
        )
    return False


def _make_identifier(name: str, prefix: Space = Space.EMPTY) -> Identifier:
    return Identifier(uuid4(), prefix, Markers.EMPTY, [], name, None, None)


_TEMPLATES = {
    ("utcnow", "from"): template("datetime.now(timezone.utc)"),
    ("utcnow", "direct"): template("datetime.datetime.now(datetime.timezone.utc)"),
    ("utcfromtimestamp", "from"): template("datetime.fromtimestamp(ts, tz=timezone.utc)"),
    ("utcfromtimestamp", "direct"): template("datetime.datetime.fromtimestamp(ts, tz=datetime.timezone.utc)"),
}


def _add_timezone_to_from_import(cu: CompilationUnit) -> CompilationUnit:
    """Add `timezone` to an existing `from datetime import ...` statement."""
    padded_stmts: List[JRightPadded] = cu.padding.statements
    for i, padded_stmt in enumerate(padded_stmts):
        stmt = padded_stmt.element
        if not isinstance(stmt, MultiImport):
            continue
        from_ = stmt.from_
        if from_ is None:
            continue
        if not (isinstance(from_, Identifier) and from_.simple_name == "datetime"):
            continue
        # Found the `from datetime import ...` — check if it has `datetime` in names
        has_datetime = any(
            imp.qualid.name.simple_name == "datetime" for imp in stmt.names
        )
        if not has_datetime:
            continue

        # Create a new Import element for `timezone`
        tz_qualid = FieldAccess(
            uuid4(), Space.EMPTY, Markers.EMPTY,
            Empty(uuid4(), Space.EMPTY, Markers.EMPTY),
            JLeftPadded(Space.EMPTY, _make_identifier("timezone"), Markers.EMPTY),
            None,
        )
        tz_import = Import(
            uuid4(), Space([], ' '), Markers.EMPTY,
            JLeftPadded(Space.EMPTY, False, Markers.EMPTY),
            tz_qualid, None,
        )

        # Append to existing names, moving trailing space from last to new element
        existing_padded_names = stmt.padding.names.padding.elements
        new_padded_names = list(existing_padded_names)
        if new_padded_names:
            last = new_padded_names[-1]
            new_padded_names[-1] = last.replace(after=Space.EMPTY)
        new_padded_names.append(JRightPadded(tz_import, last.after if existing_padded_names else Space.EMPTY, Markers.EMPTY))

        new_names_container = JContainer(
            stmt.padding.names.before,
            new_padded_names,
            stmt.padding.names.markers,
        )
        new_multi = MultiImport(
            stmt.id, stmt.prefix, stmt.markers,
            stmt.padding.from_,
            stmt.parenthesized,
            new_names_container,
        )

        new_padded_stmts = list(padded_stmts)
        new_padded_stmts[i] = padded_stmt.replace(element=new_multi)
        return cu.padding.replace(_statements=new_padded_stmts)

    return cu


@categorize(_Migration)
class DatetimeUtcNow(Recipe):
    """
    Migrate deprecated `datetime.utcnow()` and `datetime.utcfromtimestamp()`.

    Python 3.12 deprecated `datetime.utcnow()` and `datetime.utcfromtimestamp()`.
    This recipe rewrites them to `datetime.now(timezone.utc)` and
    `datetime.fromtimestamp(ts, tz=timezone.utc)` respectively.
    """

    @property
    def name(self) -> str:
        return "org.openrewrite.python.migrate.DatetimeUtcNow"

    @property
    def display_name(self) -> str:
        return "Replace deprecated `datetime.utcnow()` and `datetime.utcfromtimestamp()`"

    @property
    def description(self) -> str:
        return (
            "Replace `datetime.utcnow()` with `datetime.now(timezone.utc)` and "
            "`datetime.utcfromtimestamp(ts)` with `datetime.fromtimestamp(ts, tz=timezone.utc)` "
            "as recommended since Python 3.12."
        )

    def editor(self) -> TreeVisitor[Any, ExecutionContext]:
        class Visitor(PythonVisitor[ExecutionContext]):
            def __init__(self):
                super().__init__()
                self._needs_timezone_import = False

            def visit_compilation_unit(self, cu: CompilationUnit, p: ExecutionContext) -> J:
                self._needs_timezone_import = False
                cu = super().visit_compilation_unit(cu, p)
                if not isinstance(cu, CompilationUnit):
                    return cu
                if self._needs_timezone_import and not _has_timezone_import(cu):
                    cu = _add_timezone_to_from_import(cu)
                return cu

            def visit_method_invocation(self, method: MethodInvocation, p: ExecutionContext) -> J:
                method = super().visit_method_invocation(method, p)
                if not isinstance(method, MethodInvocation):
                    return method

                method_name = method.name.simple_name
                if method_name not in ("utcnow", "utcfromtimestamp"):
                    return method

                cu = self.cursor.first_enclosing(CompilationUnit)
                if cu is None:
                    return method

                import_style = _get_import_style(cu)
                if import_style is None:
                    return method

                if not _is_datetime_receiver(method.select, import_style):
                    return method

                if import_style == "from":
                    self._needs_timezone_import = True

                tmpl_tree = _TEMPLATES[(method_name, import_style)].get_tree()
                if not isinstance(tmpl_tree, MethodInvocation):
                    return method

                new_name = tmpl_tree.name.replace(prefix=method.name.prefix)
                tmpl_args = tmpl_tree.padding.arguments

                if method_name == "utcnow":
                    new_args = JContainer(
                        method.padding.arguments.before,
                        tmpl_args.padding.elements,
                        tmpl_args.markers,
                    )
                else:
                    # Splice original argument(s) into template args,
                    # replacing the placeholder first arg (ts) with the original
                    tmpl_padded_args = list(tmpl_args.padding.elements)
                    orig_padded_args = method.padding.arguments.padding.elements
                    if orig_padded_args:
                        tmpl_padded_args[0] = tmpl_padded_args[0].replace(
                            element=orig_padded_args[0].element
                        )
                    new_args = JContainer(
                        method.padding.arguments.before,
                        tmpl_padded_args,
                        tmpl_args.markers,
                    )

                return method.padding.replace(
                    _name=new_name,
                    _arguments=new_args,
                    _method_type=None,
                )

        return Visitor()



__path__ = __import__('pkgutil').extend_path(__path__, __name__)

# helps pytest to rewrite the assert statements in test.py
try:
    import pytest
    pytest.register_assert_rewrite("rewrite.test")
except ImportError:
    pass  # pytest not available, skip assert rewriting

from .category import CategoryDescriptor, LOWEST_PRECEDENCE, DEFAULT_PRECEDENCE, HIGHEST_PRECEDENCE
from .decorators import categorize, get_recipe_category
from .discovery import discover_recipes, discover_decorated_recipes_in_module
from .execution import ExecutionContext, DelegatingExecutionContext, InMemoryExecutionContext, \
    RecipeRunException, LargeSourceSet, InMemoryLargeSourceSet, Result
from .marketplace import RecipeMarketplace, Python
from .recipe import Recipe, ScanningRecipe, option, OptionDescriptor, RecipeDescriptor
from .markers import *
from .parser import *
from .result import *
from .style import *
from .tree import Checksum, FileAttributes, SourceFile, Tree, PrintOutputCapture, PrinterFactory
from .utils import random_id, list_find, list_map, list_map_last
from .visitor import Cursor, TreeVisitor

__all__ = [
    # Tree
    'Checksum',
    'FileAttributes',
    'SourceFile',
    'Tree',
    'PrintOutputCapture',
    'PrinterFactory',

    # Utils
    'random_id',
    'list_find',
    'list_map',
    'list_map_last',

    # Visitor
    'Cursor',
    'TreeVisitor',

    # Execution
    'ExecutionContext',
    'DelegatingExecutionContext',
    'InMemoryExecutionContext',
    'RecipeRunException',
    'LargeSourceSet',
    'InMemoryLargeSourceSet',
    'Result',

    # Recipe
    'Recipe',
    'ScanningRecipe',
    'option',
    'OptionDescriptor',
    'RecipeDescriptor',

    # Categories and Marketplace
    'CategoryDescriptor',
    'LOWEST_PRECEDENCE',
    'DEFAULT_PRECEDENCE',
    'HIGHEST_PRECEDENCE',
    'RecipeMarketplace',
    'Python',
    'categorize',
    'get_recipe_category',
    'discover_recipes',
    'discover_decorated_recipes_in_module',
    'activate',

    # Markers
    'Marker',
    'Markers',
    'SearchResult',
    'Markup',
    'MarkupWarn',
    'MarkupError',
    'MarkupInfo',
    'MarkupDebug',
    'UnknownJavaMarker',
    'ParseExceptionResult',

    # Parser
    'Parser',
    'ParserInput',
    'ParseError',
    'ParseErrorVisitor',
    'ParserBuilder',

    # Style
    'Style',
    'NamedStyles',
    'GeneralFormatStyle',
]


def activate(marketplace: RecipeMarketplace) -> None:
    """
    Install all recipes in this package into the marketplace.

    This function is called by the discovery mechanism when the
    openrewrite package is found via entry points. It installs all
    the built-in Python recipes into the provided marketplace.

    Args:
        marketplace: The RecipeMarketplace to install recipes into
    """
    from rewrite.decorators import get_recipe_category
    from rewrite.python.recipes import RemovePass

    # Install all Python recipes with their categories
    category = get_recipe_category(RemovePass)
    if category is not None:
        marketplace.install(RemovePass, category)



# Copyright 2025 the original author or authors.
#
# Licensed under the Moderne Source Available License (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://docs.moderne.io/licensing/moderne-source-available-license
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
ReorderMethodArguments recipe for Python that delegates to Java's ReorderMethodArguments.

This recipe reorders arguments in method invocations matching a pattern.
"""

from typing import Any, List, Optional

from rewrite import ExecutionContext, Recipe, TreeVisitor, option
from rewrite.rpc.java_recipe import prepare_java_recipe, JavaRecipeVisitor, PreparedJavaRecipe


class ReorderMethodArguments(Recipe):
    """
    Reorder arguments in method invocations matching a pattern.

    This recipe delegates to Java's org.openrewrite.java.ReorderMethodArguments recipe.

    Example:
        ReorderMethodArguments(
            method_pattern="datetime.datetime replace(..)",
            new_parameter_names=["year", "month", "day"],
            old_parameter_names=["day", "month", "year"]
        )
    """

    method_pattern: str = option(
        display_name="Method pattern",
        description="A method pattern that matches method invocations to reorder.",
        example="datetime.datetime replace(..)"
    )

    new_parameter_names: List[str] = option(
        display_name="New parameter names",
        description="The parameter names in the desired order.",
        example='["year", "month", "day"]'
    )

    old_parameter_names: List[str] = option(
        display_name="Old parameter names",
        description="The current parameter names (optional if using positional reordering).",
        required=False
    )

    def __init__(
        self,
        method_pattern: str,
        new_parameter_names: List[str],
        old_parameter_names: Optional[List[str]] = None
    ):
        self.method_pattern = method_pattern
        self.new_parameter_names = new_parameter_names
        self.old_parameter_names = old_parameter_names
        self._prepared_recipe: Optional[PreparedJavaRecipe] = None

    @property
    def name(self) -> str:
        return "org.openrewrite.python.ReorderMethodArguments"

    @property
    def display_name(self) -> str:
        return "Reorder method arguments"

    @property
    def description(self) -> str:
        return "Reorder arguments in method invocations matching a pattern."

    def editor(self) -> TreeVisitor[Any, ExecutionContext]:
        if self._prepared_recipe is None:
            options = {
                "methodPattern": self.method_pattern,
                "newParameterNames": self.new_parameter_names
            }
            if self.old_parameter_names:
                options["oldParameterNames"] = self.old_parameter_names

            self._prepared_recipe = prepare_java_recipe(
                "org.openrewrite.java.ReorderMethodArguments",
                options
            )

        return JavaRecipeVisitor(self._prepared_recipe)
